@@ -21,12 +21,13 @@
 #define PLUGINTEST_H
 
 #include <QCoreApplication>
+#include <plasma/dataengine.h>
 
 class PluginTest : public QObject
 {
     Q_OBJECT
 public:
-    PluginTest() {}
+    PluginTest();
 
 private Q_SLOTS:
     void listEngines();
@@ -37,6 +38,17 @@ private Q_SLOTS:
     void loadDataEngine();
 
 private:
+    bool m_buildonly;
+};
+
+class EngineTest : public QObject
+{
+    Q_OBJECT
+public:
+    EngineTest() {}
+
+public Q_SLOTS:
+    void dataUpdated(const QString &name, const Plasma::DataEngine::Data &data);
 };
 
 #endif
