@@ -1,5 +1,6 @@
 /*
     Copyright (C) 2013 Mark Gaiser <markg85@gmail.com>
+    Copyright (C) 2016 Martin Klapetek <mklapetek@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -126,7 +127,7 @@ void Calendar::setDays(int days)
     }
 }
 
-int Calendar::weeks()
+int Calendar::weeks() const
 {
     return m_weeks;
 }
@@ -140,7 +141,7 @@ void Calendar::setWeeks(int weeks)
     }
 }
 
-int Calendar::firstDayOfWeek()
+int Calendar::firstDayOfWeek() const
 {
     // QML has Sunday as 0, so we need to accomodate here
     return m_firstDayOfWeek == 7 ? 0 : m_firstDayOfWeek;
@@ -192,7 +193,7 @@ QString Calendar::monthName() const
     // locale and take the month name from that.
     //
     // See https://bugs.kde.org/show_bug.cgi?id=353715
-    const QString lang = QLocale().uiLanguages().first();
+    const QString lang = QLocale().uiLanguages().at(0);
     // If lang is empty, it will create just a system locale
     QLocale langLocale(lang);
     return langLocale.standaloneMonthName(m_displayedDate.month());
