@@ -44,7 +44,7 @@ public:
         yearNumber
     };
 
-    explicit DaysModel(QObject *parent = 0);
+    explicit DaysModel(QObject *parent = nullptr);
     virtual ~DaysModel();
     void setSourceData(QList<DayData> *data);
     int rowCount(const QModelIndex &parent) const Q_DECL_OVERRIDE;
@@ -70,8 +70,8 @@ private Q_SLOTS:
 private:
     QModelIndex indexForDate(const QDate &date);
 
-    EventPluginsManager *m_pluginsManager;
-    QList<DayData> *m_data;
+    EventPluginsManager *m_pluginsManager = nullptr;
+    QList<DayData> *m_data = nullptr;
     QList<QObject*> m_qmlData;
     QDate m_lastRequestedAgendaDate;
     QList<CalendarEvents::CalendarEventsPlugin*> m_eventPlugins;
