@@ -24,8 +24,6 @@
 #include <QQmlEngine>
 #include <QTimer>
 
-#include <QDebug>
-
 namespace Plasma
 {
 
@@ -55,8 +53,9 @@ void SortFilterModel::syncRoleNames()
 
     m_roleIds.clear();
     const QHash<int, QByteArray> rNames = roleNames();
+    m_roleIds.reserve(rNames.count());
     for (auto i = rNames.constBegin(); i != rNames.constEnd(); ++i) {
-        m_roleIds[i.value()] = i.key();
+        m_roleIds[QString::fromUtf8(i.value())] = i.key();
     }
 
     setRoleNames(sourceModel()->roleNames());
@@ -64,12 +63,9 @@ void SortFilterModel::syncRoleNames()
     setSortRole(m_sortRole);
 }
 
-int SortFilterModel::roleNameToId(const QString &name)
+int SortFilterModel::roleNameToId(const QString &name) const
 {
-    if (!m_roleIds.contains(name)) {
-        return Qt::DisplayRole;
-    }
-    return m_roleIds.value(name);
+    return m_roleIds.value(name, Qt::DisplayRole);
 }
 
 void SortFilterModel::setModel(QAbstractItemModel *model)
@@ -114,7 +110,7 @@ void SortFilterModel::setFilterRegExp(const QString &exp)
         return;
     }
     QSortFilterProxyModel::setFilterRegExp(QRegExp(exp, Qt::CaseInsensitive));
-    filterRegExpChanged(exp);
+    Q_EMIT filterRegExpChanged(exp);
 }
 
 QString SortFilterModel::filterRegExp() const
@@ -129,7 +125,7 @@ void SortFilterModel::setFilterString(const QString &filterString)
     }
     m_filterString = filterString;
     QSortFilterProxyModel::setFilterFixedString(filterString);
-    filterStringChanged(filterString);
+    Q_EMIT filterStringChanged(filterString);
 }
 
 QString SortFilterModel::filterString() const
@@ -153,8 +149,9 @@ void SortFilterModel::setFilterCallback(const QJSValue& callback)
     }
 
     m_filterCallback = callback;
+    invalidateFilter();
 
-    emit filterCallbackChanged(callback);
+    Q_EMIT filterCallbackChanged(callback);
 }
 
 void SortFilterModel::setFilterRole(const QString &role)
@@ -196,7 +193,7 @@ QVariantMap SortFilterModel::get(int row) const
 
     const QHash<int, QByteArray> rNames = roleNames();
     for (auto i = rNames.begin(); i != rNames.end(); ++i) {
-        hash[i.value()] = data(idx, i.key());
+        hash[QString::fromUtf8(i.value())] = data(idx, i.key());
     }
 
     return hash;
@@ -225,7 +222,7 @@ DataModel::DataModel(QObject *parent)
 {
     //There is one reserved role name: DataEngineSource
     m_roleNames[m_maxRoleId] = QByteArrayLiteral("DataEngineSource");
-    m_roleIds["DataEngineSource"] = m_maxRoleId;
+    m_roleIds[QStringLiteral("DataEngineSource")] = m_maxRoleId;
     ++m_maxRoleId;
 
     setObjectName(QStringLiteral("DataModel"));
@@ -259,7 +256,7 @@ void DataModel::dataUpdated(const QString &sourceName, const QVariantMap &data)
                 QVariant value = m_dataSource->data()->value(key);
                 if (value.isValid() && value.canConvert<Plasma::DataEngine::Data>()) {
                     Plasma::DataEngine::Data data = value.value<Plasma::DataEngine::Data>();
-                    data["DataEngineSource"] = key;
+                    data[QStringLiteral("DataEngineSource")] = key;
                     list.append(data);
                 }
             }
@@ -437,25 +434,39 @@ void DataModel::setItems(const QString &sourceName, const QVariantList &list)
 
 void DataModel::removeSource(const QString &sourceName)
 {
-    //FIXME: this could be way more efficient by not resetting the whole model
     //FIXME: find a way to remove only the proper things also in the case where sources are items
 
     if (m_keyRoleFilter.isEmpty()) {
         //source name in the map, linear scan
         for (int i = 0; i < m_items.value(QString()).count(); ++i) {
-            if (m_items.value(QString())[i].value<QVariantMap>().value("DataEngineSource") == sourceName) {
-                beginResetModel();
+            if (m_items.value(QString())[i].value<QVariantMap>().value(QStringLiteral("DataEngineSource")) == sourceName) {
+                beginRemoveRows(QModelIndex(), i, i);
                 m_items[QString()].remove(i);
-                endResetModel();
+                endRemoveRows();
                 break;
             }
         }
     } else {
-        //source name as key of the map
         if (m_items.contains(sourceName)) {
-            beginResetModel();
+            //At what row number the first item associated to this source starts
+            int sourceIndex = 0;
+            for (auto i = m_items.constBegin(); i != m_items.constEnd(); ++i) {
+                if (i.key() == sourceName) {
+                    break;
+                }
+                sourceIndex += i.value().count();
+            }
+
+            //source name as key of the map
+
+            int count = m_items.value(sourceName).count();
+            if (count > 0) {
+                beginRemoveRows(QModelIndex(), sourceIndex, sourceIndex + count - 1);
+            }
             m_items.remove(sourceName);
-            endResetModel();
+            if (count > 0) {
+                endRemoveRows();
+            }
         }
     }
 }
@@ -487,7 +498,7 @@ QVariant DataModel::data(const QModelIndex &index, int role) const
     if (!m_keyRoleFilter.isEmpty() && m_roleNames.value(role) == "DataEngineSource") {
         return source;
     } else {
-        return m_items.value(source).value(actualRow).value<QVariantMap>().value(m_roleNames.value(role));
+        return m_items.value(source).value(actualRow).value<QVariantMap>().value(QString::fromUtf8(m_roleNames.value(role)));
     }
 }
 
@@ -543,19 +554,11 @@ QVariantMap DataModel::get(int row) const
 
     const QHash<int, QByteArray> rNames = roleNames();
     for (auto i = rNames.constBegin(); i != rNames.constEnd(); ++i) {
-        map[i.value()] = data(idx, i.key());
+        map[QString::fromUtf8(i.value())] = data(idx, i.key());
     }
 
     return map;
 }
 
-int DataModel::roleNameToId(const QString &name)
-{
-    if (!m_roleIds.contains(name)) {
-        return -1;
-    }
-    return m_roleIds.value(name);
-}
-
 }
 
