@@ -39,7 +39,7 @@ IconItem::IconItem(QQuickItem *parent)
     : QQuickItem(parent),
       m_svgIcon(0),
       m_status(Plasma::Svg::Normal),
-      m_smooth(false),
+      m_smooth(true),
       m_active(false),
       m_animated(true),
       m_usesPlasmaTheme(true),
@@ -68,13 +68,7 @@ IconItem::IconItem(QQuickItem *parent)
             this, &IconItem::updateImplicitSize);
 
     connect(this, &QQuickItem::enabledChanged,
-            this, &IconItem::enabledChanged);
-
-    connect(this, &QQuickItem::windowChanged,
-            this, &IconItem::schedulePixmapUpdate);
-
-    connect(this, &IconItem::overlaysChanged,
-            this, &IconItem::schedulePixmapUpdate);
+            this, &IconItem::onEnabledChanged);
 
     connect(this, &IconItem::implicitWidthChanged, this, &IconItem::implicitWidthChanged2);
     connect(this, &IconItem::implicitHeightChanged, this, &IconItem::implicitHeightChanged2);
@@ -145,6 +139,8 @@ void IconItem::setSource(const QVariant &source)
         return;
     }
 
+    const bool oldValid = isValid();
+
     m_source = source;
     QString sourceString = source.toString();
 
@@ -154,11 +150,17 @@ void IconItem::setSource(const QVariant &source)
     }
 
     if (!sourceString.isEmpty()) {
-        //If a url in the form file:// is passed, take the image pointed by that from disk
+        // If a file:// URL or a absolute path is passed, take the image pointed by that from disk
+        QString localFile;
         if (sourceString.startsWith(QLatin1String("file:"))) {
-            const QUrl url(sourceString);
+            localFile = QUrl(sourceString).toLocalFile();
+        } else if (sourceString.startsWith(QLatin1Char('/'))) {
+            localFile = sourceString;
+        }
+
+        if (!localFile.isEmpty()) {
             m_icon = QIcon();
-            m_imageIcon = QImage(url.toLocalFile());
+            m_imageIcon = QImage(localFile);
             m_svgIconName.clear();
             delete m_svgIcon;
             m_svgIcon = 0;
@@ -173,7 +175,7 @@ void IconItem::setSource(const QVariant &source)
 
             if (m_usesPlasmaTheme) {
                 //try as a svg icon from plasma theme
-                m_svgIcon->setImagePath(QLatin1String("icons/") + sourceString.section('-', 0, 0));
+                m_svgIcon->setImagePath(QLatin1String("icons/") + sourceString.section(QLatin1Char('-'), 0, 0));
                 m_svgIcon->setContainsMultipleImages(true);
             }
 
@@ -240,7 +242,10 @@ void IconItem::setSource(const QVariant &source)
     updateImplicitSize();
 
     emit sourceChanged();
-    emit validChanged();
+
+    if (isValid() != oldValid) {
+        emit validChanged();
+    }
 }
 
 QVariant IconItem::source() const
@@ -275,6 +280,7 @@ void IconItem::setOverlays(const QStringList &overlays)
         return;
     }
     m_overlays = overlays;
+    schedulePixmapUpdate();
     emit overlaysChanged();
 }
 
@@ -476,7 +482,9 @@ QSGNode* IconItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *update
             delete oldNode;
 
             QSGTexture *source = window()->createTextureFromImage(m_iconPixmap.toImage());
+            source->setFiltering(m_smooth ? QSGTexture::Linear : QSGTexture::Nearest);
             QSGTexture *target = window()->createTextureFromImage(m_oldIconPixmap.toImage());
+            target->setFiltering(m_smooth ? QSGTexture::Linear : QSGTexture::Nearest);
             animatingNode = new FadingNode(source, target);
             m_sizeChanged = true;
             m_textureChanged = false;
@@ -520,7 +528,7 @@ void IconItem::valueChanged(const QVariant &value)
     update();
 }
 
-void IconItem::enabledChanged()
+void IconItem::onEnabledChanged()
 {
     m_allowNextAnimation = true;
     schedulePixmapUpdate();
@@ -557,6 +565,7 @@ void IconItem::loadPixmap()
         update();
         return;
     } else if (m_svgIcon) {
+        m_svgIcon->setDevicePixelRatio(window() ? window()->devicePixelRatio() : qApp->devicePixelRatio());
         m_svgIcon->resize(size, size);
         if (!m_svgIconName.isEmpty() && m_svgIcon->hasElement(m_svgIconName)) {
             result = m_svgIcon->pixmap(m_svgIconName);
@@ -619,7 +628,15 @@ void IconItem::loadPixmap()
     }
 
     //don't animate initial setting
-    if ((m_animated || m_allowNextAnimation) && !m_oldIconPixmap.isNull() && !m_sizeChanged && !m_blockNextAnimation) {
+    bool animated = (m_animated || m_allowNextAnimation) && !m_oldIconPixmap.isNull() && !m_sizeChanged && !m_blockNextAnimation;
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 9, 0))
+    if (QQuickWindow::sceneGraphBackend() == QLatin1String("software")) {
+        animated = false;
+    }
+#endif
+
+    if (animated) {
         m_animValue = 0.0;
         m_animation->setStartValue((qreal)0);
         m_animation->setEndValue((qreal)1);
@@ -637,6 +654,8 @@ void IconItem::itemChange(ItemChange change, const ItemChangeData &value)
 {
     if (change == ItemVisibleHasChanged && value.boolValue) {
         m_blockNextAnimation = true;
+    } else if (change == ItemSceneChange && value.window) {
+        schedulePixmapUpdate();
     }
 
     QQuickItem::itemChange(change, value);
