@@ -60,9 +60,15 @@ class IconItem : public QQuickItem
      */
     Q_PROPERTY(Plasma::Theme::ColorGroup colorGroup READ colorGroup WRITE setColorGroup NOTIFY colorGroupChanged)
 
+    /**
+      * Specifies the overlay(s) for this icon
+      */
+    Q_PROPERTY(QStringList overlays READ overlays WRITE setOverlays NOTIFY overlaysChanged)
+
     /**
      * See QQuickItem::smooth
      */
+    //KF6 Remove, this just shadows QQuickItem::smooth
     Q_PROPERTY(bool smooth READ smooth WRITE setSmooth NOTIFY smoothChanged)
 
     /**
@@ -71,6 +77,17 @@ class IconItem : public QQuickItem
      */
     Q_PROPERTY(bool active READ isActive WRITE setActive NOTIFY activeChanged)
 
+    /**
+     * Sets the image in a selected status.
+     * Svgs can be colored with system color themes, if the status is selected,
+     * the TextColor will become HighlightedText color and BackgroundColor
+     * will become HighlightColor, making the svg graphics (for instance an icon)
+     * will look correct together selected text
+     * @see Plasma::Svg::status
+     * @since 5.23
+     */
+    Q_PROPERTY(Plasma::Svg::Status status READ status WRITE setStatus NOTIFY statusChanged)
+
     /**
      * If set, icon will blend when the source is changed
      */
@@ -81,6 +98,11 @@ class IconItem : public QQuickItem
      */
     Q_PROPERTY(bool usesPlasmaTheme READ usesPlasmaTheme WRITE setUsesPlasmaTheme NOTIFY usesPlasmaThemeChanged)
 
+    /**
+     * If set, icon will round the painted size to defined icon sizes. Default is true.
+     */
+    Q_PROPERTY(bool roundToIconSize READ roundToIconSize WRITE setRoundToIconSize NOTIFY roundToIconSizeChanged)
+
     /**
      * True if a valid icon is set. False otherwise.
      */
@@ -98,9 +120,12 @@ class IconItem : public QQuickItem
      */
     Q_PROPERTY(int paintedHeight READ paintedHeight NOTIFY paintedSizeChanged)
 
-public:
+    Q_PROPERTY(int implicitHeight READ implicitHeight WRITE setImplicitHeight2 NOTIFY implicitHeightChanged2)
 
-    IconItem(QQuickItem *parent = 0);
+    Q_PROPERTY(int implicitWidth READ implicitWidth WRITE setImplicitWidth2 NOTIFY implicitWidthChanged2)
+
+public:
+    explicit IconItem(QQuickItem *parent = nullptr);
     ~IconItem();
 
     void setSource(const QVariant &source);
@@ -109,6 +134,9 @@ public:
     void setColorGroup(Plasma::Theme::ColorGroup group);
     Plasma::Theme::ColorGroup colorGroup() const;
 
+    void setOverlays(const QStringList &overlays);
+    QStringList overlays() const;
+
     bool isActive() const;
     void setActive(bool active);
 
@@ -121,11 +149,20 @@ public:
     bool usesPlasmaTheme() const;
     void setUsesPlasmaTheme(bool usesPlasmaTheme);
 
+    bool roundToIconSize() const;
+    void setRoundToIconSize(bool roundToIconSize);
+
     bool isValid() const;
 
     int paintedWidth() const;
     int paintedHeight() const;
 
+    void setStatus(Plasma::Svg::Status status);
+    Plasma::Svg::Status status() const;
+
+    void setImplicitHeight2(int height);
+    void setImplicitWidth2(int height);
+
     void updatePolish() Q_DECL_OVERRIDE;
     QSGNode* updatePaintNode(QSGNode * oldNode, UpdatePaintNodeData * updatePaintNodeData) Q_DECL_OVERRIDE;
 
@@ -136,23 +173,30 @@ public:
     void componentComplete() Q_DECL_OVERRIDE;
 
 Q_SIGNALS:
+    void overlaysChanged();
     void activeChanged();
     void sourceChanged();
     void smoothChanged();
     void animatedChanged();
     void usesPlasmaThemeChanged();
+    void roundToIconSizeChanged();
     void validChanged();
     void colorGroupChanged();
     void paintedSizeChanged();
+    void statusChanged();
+    void implicitHeightChanged2();
+    void implicitWidthChanged2();
 
 private Q_SLOTS:
     void schedulePixmapUpdate();
     void animationFinished();
     void valueChanged(const QVariant &value);
-    void enabledChanged();
+    void onEnabledChanged();
 
 private:
     void loadPixmap();
+    QSize paintedSize(const QSizeF &containerSize = QSizeF()) const;
+    void updateImplicitSize();
 
     //all the ways we can set an source. Only one of them will be valid
     QIcon m_icon;
@@ -162,6 +206,7 @@ private:
     QImage m_imageIcon;
     //this contains the raw variant it was passed
     QVariant m_source;
+    Plasma::Svg::Status m_status;
 
     QSizeF m_implicitSize;
 
@@ -169,20 +214,25 @@ private:
     bool m_active;
     bool m_animated;
     bool m_usesPlasmaTheme;
+    bool m_roundToIconSize;
 
     bool m_textureChanged;
     bool m_sizeChanged;
     bool m_allowNextAnimation;
+    bool m_blockNextAnimation;
+    bool m_implicitHeightSetByUser;
+    bool m_implicitWidthSetByUser;
 
     QPixmap m_iconPixmap;
     QPixmap m_oldIconPixmap;
 
+    QStringList m_overlays;
+
     Plasma::Theme::ColorGroup m_colorGroup;
 
     //animation on pixmap change
     QPropertyAnimation *m_animation;
     qreal m_animValue;
-
 };
 
 #endif
