@@ -68,7 +68,7 @@ class SvgItem : public QQuickItem
 public:
 /// @cond INTERNAL_DOCS
 
-    SvgItem(QQuickItem *parent = 0);
+    explicit SvgItem(QQuickItem *parent = nullptr);
     ~SvgItem();
 
     void setElementId(const QString &elementID);
@@ -106,7 +106,6 @@ private:
     QString m_elementID;
     bool m_smooth;
     bool m_textureChanged;
-    Units m_units;
     QImage m_image;
 };
 }
