@@ -30,7 +30,7 @@
 #include <kwindoweffects.h>
 #include <KDirWatch>
 
-ToolTipDialog *ToolTip::s_dialog = 0;
+ToolTipDialog *ToolTip::s_dialog = nullptr;
 int ToolTip::s_dialogUsers  = 0;
 
 ToolTip::ToolTip(QQuickItem *parent)
@@ -38,6 +38,7 @@ ToolTip::ToolTip(QQuickItem *parent)
       m_tooltipsEnabledGlobally(false),
       m_containsMouse(false),
       m_location(Plasma::Types::Floating),
+      m_textFormat(Qt::AutoText),
       m_active(true),
       m_interactive(false),
       m_usingDialog(false)
@@ -49,9 +50,9 @@ ToolTip::ToolTip(QQuickItem *parent)
     m_showTimer->setSingleShot(true);
     connect(m_showTimer, &QTimer::timeout, this, &ToolTip::showToolTip);
 
-    settingsChanged();
+    loadSettings();
 
-    const QString configFile = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/') + "plasmarc";
+    const QString configFile = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QStringLiteral("/plasmarc");
     KDirWatch::self()->addFile(configFile);
     QObject::connect(KDirWatch::self(), SIGNAL(created(QString)), this, SLOT(settingsChanged()));
     QObject::connect(KDirWatch::self(), SIGNAL(dirty(QString)), this, SLOT(settingsChanged()));
@@ -59,22 +60,30 @@ ToolTip::ToolTip(QQuickItem *parent)
 
 ToolTip::~ToolTip()
 {
+    if (s_dialog && s_dialog->owner() == this) {
+        s_dialog->setVisible(false);
+    }
+
     if (m_usingDialog) {
         --s_dialogUsers;
     }
 
     if (s_dialogUsers == 0) {
         delete s_dialog;
-        s_dialog = 0;
+        s_dialog = nullptr;
     }
 }
 
 void ToolTip::settingsChanged()
 {
-    KConfig config("plasmarc");
-    KConfigGroup cg(&config, "PlasmaToolTips");
+    KSharedConfig::openConfig(QStringLiteral("plasmarc"))->reparseConfiguration();
+    loadSettings();
+}
 
-    m_interval = cg.readEntry("Delay", 700);
+void ToolTip::loadSettings()
+{
+    KConfigGroup cfg = KConfigGroup(KSharedConfig::openConfig(QStringLiteral("plasmarc")), "PlasmaToolTips");
+    m_interval = cfg.readEntry("Delay", 700);
     m_tooltipsEnabledGlobally = (m_interval > 0);
 }
 
@@ -104,6 +113,10 @@ void ToolTip::setMainItem(QQuickItem *mainItem)
         m_mainItem = mainItem;
 
         emit mainItemChanged();
+
+        if (!isValid() && s_dialog && s_dialog->owner() == this) {
+            s_dialog->setVisible(false);
+        }
     }
 }
 
@@ -119,9 +132,8 @@ void ToolTip::showToolTip()
         setMainItem(dlg->loadDefaultItem());
     }
 
-    if (dlg->mainItem()) {
-        dlg->mainItem()->setVisible(false);
-    }
+    // Unset the dialog's old contents before reparenting the dialog.
+    dlg->setMainItem(nullptr);
 
     Plasma::Types::Location location = m_location;
     if (m_location == Plasma::Types::Floating) {
@@ -140,41 +152,12 @@ void ToolTip::showToolTip()
         mainItem()->setVisible(true);
     }
 
-    //heuristics for knowing the diration
-    Plasma::Types::Direction dir;
-    QPoint pos = mapToScene(QPoint(0, 0)).toPoint();
-
-    if (window() && window()->screen()) {
-        pos = window()->mapToGlobal(pos);
-    }
-    QPoint popupPos = dlg->popupPosition(this, dlg->size());
-
-    if (location == Plasma::Types::Floating) {
-        dir = Plasma::Types::Up;
-
-        if (pos.y() + height() <= popupPos.y()) {
-            dir = Plasma::Types::Down;
-        } else if (pos.x() + width() <= popupPos.x()) {
-            dir = Plasma::Types::Right;
-        } else if (pos.y() >= popupPos.y() + dlg->height()) {
-            dir = Plasma::Types::Up;
-        } else if (pos.x() >= popupPos.x() + dlg->width()) {
-            dir = Plasma::Types::Left;
-        }
-    } else {
-        dir = Plasma::locationToDirection(location);
-        if (dir == Plasma::Types::Left && (pos.x() + width() <= popupPos.x())) {
-            dir = Plasma::Types::Right;
-        } else if (dir == Plasma::Types::Up && (pos.y() + height() <= popupPos.y())) {
-            dir = Plasma::Types::Down;
-        }
-    }
-
+    dlg->setOwner(this);
     dlg->setLocation(location);
-    dlg->setMainItem(mainItem());
     dlg->setVisualParent(this);
+    dlg->setMainItem(mainItem());
     dlg->setInteractive(m_interactive);
-    QMetaObject::invokeMethod(dlg, "show", Qt::QueuedConnection);
+    dlg->setVisible(true);
 }
 
 QString ToolTip::mainText() const
@@ -190,6 +173,10 @@ void ToolTip::setMainText(const QString &mainText)
 
     m_mainText = mainText;
     emit mainTextChanged();
+
+    if (!isValid() && s_dialog && s_dialog->owner() == this) {
+        s_dialog->setVisible(false);
+    }
 }
 
 QString ToolTip::subText() const
@@ -205,6 +192,25 @@ void ToolTip::setSubText(const QString &subText)
 
     m_subText = subText;
     emit subTextChanged();
+
+    if (!isValid() && s_dialog && s_dialog->owner() == this) {
+        s_dialog->setVisible(false);
+    }
+}
+
+int ToolTip::textFormat() const
+{
+    return m_textFormat;
+}
+
+void ToolTip::setTextFormat(int format)
+{
+    if (m_textFormat == format) {
+        return;
+    }
+
+    m_textFormat = format;
+    emit textFormatChanged();
 }
 
 Plasma::Types::Location ToolTip::location() const
@@ -247,6 +253,7 @@ void ToolTip::setInteractive(bool interactive)
 
 void ToolTip::hideToolTip()
 {
+    m_showTimer->stop();
     tooltipDialogInstance()->dismiss();
 }
 
@@ -306,28 +313,36 @@ void ToolTip::setContainsMouse(bool contains)
 
 void ToolTip::hoverEnterEvent(QHoverEvent *event)
 {
+    Q_UNUSED(event)
     setContainsMouse(true);
 
     if (!m_tooltipsEnabledGlobally) {
         return;
     }
 
+    if (!isValid()) {
+        return;
+    }
+
     if (tooltipDialogInstance()->isVisible()) {
         // We signal the tooltipmanager that we're "potentially interested,
         // and ask to keep it open for a bit, so other items get the chance
         // to update the content before the tooltip hides -- this avoids
         // flickering
-        tooltipDialogInstance()->keepalive();
-        //FIXME: showToolTip needs to be renamed in sync or something like that
-        showToolTip();
-    } else if (m_mainItem ||
-               (!mainText().isEmpty() && !subText().isEmpty())) {
+        // It need to be considered only when other items can deal with tooltip area
+        if (m_active) {
+            tooltipDialogInstance()->keepalive();
+            //FIXME: showToolTip needs to be renamed in sync or something like that
+            showToolTip();
+        }
+    } else {
         m_showTimer->start(m_interval);
     }
 }
 
 void ToolTip::hoverLeaveEvent(QHoverEvent *event)
 {
+    Q_UNUSED(event)
     setContainsMouse(false);
     m_showTimer->stop();
 }
@@ -337,3 +352,7 @@ bool ToolTip::childMouseEventFilter(QQuickItem *item, QEvent *event)
     return QQuickItem::childMouseEventFilter(item, event);
 }
 
+bool ToolTip::isValid() const
+{
+    return m_mainItem || !mainText().isEmpty() || !subText().isEmpty();
+}
