@@ -22,9 +22,7 @@
 #include <QQmlEngine>
 #include <QQuickItem>
 #include <QDebug>
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
 #include <QPlatformSurfaceEvent>
-#endif
 
 #include <kdeclarative/qmlobjectsharedengine.h>
 #include <KWindowSystem>
@@ -34,14 +32,10 @@ ToolTipDialog::ToolTipDialog(QQuickItem  *parent)
       m_qmlObject(0),
       m_hideTimeout(4000),
       m_interactive(false),
-      m_owner(Q_NULLPTR)
+      m_owner(nullptr)
 {
-    Qt::WindowFlags flags = Qt::ToolTip;
-    if (KWindowSystem::isPlatformX11()) {
-        flags = flags | Qt::BypassWindowManagerHint;
-    }
-    setFlags(flags);
     setLocation(Plasma::Types::Floating);
+    setType(Dialog::WindowType::Tooltip);
 
     m_showTimer = new QTimer(this);
     m_showTimer->setSingleShot(true);
@@ -63,8 +57,8 @@ QQuickItem *ToolTipDialog::loadDefaultItem()
     if (!m_qmlObject->rootObject()) {
         //HACK: search our own import
         foreach (const QString &path, m_qmlObject->engine()->importPathList()) {
-            if (QFile::exists(path + "/org/kde/plasma/core")) {
-                m_qmlObject->setSource(QUrl::fromLocalFile(path + "/org/kde/plasma/core/private/DefaultToolTip.qml"));
+            if (QFile::exists(path + QStringLiteral("/org/kde/plasma/core"))) {
+                m_qmlObject->setSource(QUrl::fromLocalFile(path + QStringLiteral("/org/kde/plasma/core/private/DefaultToolTip.qml")));
                 break;
             }
         }
@@ -102,15 +96,6 @@ bool ToolTipDialog::event(QEvent *e)
         dismiss();
     }
 
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
-    if (e->type() == QEvent::PlatformSurface) {
-        auto pe = static_cast<QPlatformSurfaceEvent*>(e);
-        if (pe->surfaceEventType() == QPlatformSurfaceEvent::SurfaceCreated) {
-            KWindowSystem::setType(winId(), NET::Tooltip);
-        }
-    }
-#endif
-
     bool ret = Dialog::event(e);
     Qt::WindowFlags flags = Qt::ToolTip | Qt::WindowDoesNotAcceptFocus | Qt::WindowStaysOnTopHint;
     if (KWindowSystem::isPlatformX11()) {
