@@ -1,47 +1,49 @@
 /*
-    SPDX-FileCopyrightText: 2006-2007 Aaron Seigo <:wqaseigo@kde.org>
-    SPDX-FileCopyrightText: 2007 Riccardo Iaconelli <riccardo@kde.org>
-    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
+ *   Copyright 2006-2007 by Aaron Seigo <aseigo@kde.org>
+ *   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>
+ *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
 
-    SPDX-License-Identifier: LGPL-2.0-or-later
-*/
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU Library General Public License as
+ *   published by the Free Software Foundation; either version 2, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details
+ *
+ *   You should have received a copy of the GNU Library General Public
+ *   License along with this program; if not, write to the
+ *   Free Software Foundation, Inc.,
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
 
 #ifndef PLASMA_APPLET_H
 #define PLASMA_APPLET_H
 
-#include <QAction>
-#include <QKeySequence>
 #include <QObject>
-#include <QQmlListProperty>
+#include <QKeySequence>
 #include <QUrl>
 
-#include <KConfigGroup>
-#include <plasma/plasma_export.h>
+#include <kconfiggroup.h>
+#include <kplugininfo.h>
 
 #include <plasma/plasma.h>
+#include <plasma/version.h>
+#include <plasma/framesvg.h>
 
-namespace KPackage
-{
-class Package;
-}
-namespace PlasmaQuick
-{
-class AppletQuickItem;
-class ConfigViewPrivate;
-class ConfigModelPrivate;
-class ConfigModel;
-class ConfigView;
-};
-class DeclarativeAppletScript;
-#include <KPluginFactory>
+#include <KPackage/Package>
 
+class KActionCollection;
 class KConfigLoader;
-class KConfigPropertyMap;
 
 namespace Plasma
 {
+
 class AppletPrivate;
 class Containment;
+class DataEngine;
 class Package;
 
 /**
@@ -52,8 +54,10 @@ class Package;
  * Applet provides several important roles for add-ons widgets in Plasma.
  *
  * First, it is the base class for the plugin system and therefore is the
- * interface to applets for host applications.
- * Background painting (allowing for consistent and complex
+ * interface to applets for host applications. It also handles the life time
+ * management of data engines (e.g. all data engines accessed via
+ * Applet::dataEngine(const QString&) are properly deref'd on Applet
+ * destruction), background painting (allowing for consistent and complex
  * look and feel in just one line of code for applets), loading and starting
  * of scripting support for each applet, providing access to the associated
  * plasmoid package (if any) and access to configuration data.
@@ -63,214 +67,58 @@ class Package;
 class PLASMA_EXPORT Applet : public QObject
 {
     Q_OBJECT
-    /**
-     * Applet id: is unique in the whole Plasma session and will never change across restarts
-     */
-    Q_PROPERTY(uint id READ id CONSTANT FINAL)
-
-    /**
-     * User friendly title for the plasmoid: it's the localized applet name by default
-     */
-    Q_PROPERTY(QString title READ title WRITE setTitle NOTIFY titleChanged FINAL)
-
-    /**
-     * Icon to represent the plasmoid
-     */
-    Q_PROPERTY(QString icon READ icon WRITE setIcon NOTIFY iconChanged FINAL)
-
-    // TODO KF6 toolTipMainText toolTipSubText toolTipTextFormat toolTipItem: need to either be here or some other kind of attached property
-
-    /**
-     * The current form factor the applet is being displayed in.
-     *
-     * @see Plasma::FormFactor
-     */
-    Q_PROPERTY(Plasma::Types::FormFactor formFactor READ formFactor NOTIFY formFactorChanged)
-
-    /**
-     * The location of the scene which is displaying applet.
-     *
-     * @see Plasma::Types::Location
-     */
-    Q_PROPERTY(Plasma::Types::Location location READ location NOTIFY locationChanged)
-
-    /**
-     * Status of the plasmoid: useful to instruct the shell if this plasmoid is requesting attention, if is accepting input, or if is in an idle, inactive state
-     */
     Q_PROPERTY(Plasma::Types::ItemStatus status READ status WRITE setStatus NOTIFY statusChanged)
-
-    /**
-     * The immutability of the Corona.
-     * Tells the applet whether it should allow for any modification by the user.
-     */
     Q_PROPERTY(Plasma::Types::ImmutabilityType immutability READ immutability WRITE setImmutability NOTIFY immutabilityChanged)
-
-    /**
-     * Whether the Corona is immutable. The plasmoid implementation should avoid allowing "dangerous" modifications from the user when in an immutable mode
-     *
-     * This is true when immutability is not Mutable
-     */
-    Q_PROPERTY(bool immutable READ immutable NOTIFY immutabilityChanged)
-
-    /**
-     * Display hints that come from the containment that suggest the applet how to look and behave.
-     * TODO: only in containment?
-     */
-    Q_PROPERTY(Plasma::Types::ContainmentDisplayHints containmentDisplayHints READ containmentDisplayHints NOTIFY containmentDisplayHintsChanged)
-
-    /**
-     * True if the applet should show a busy status, for instance doing
-     * some network operation
-     */
+    Q_PROPERTY(Plasma::Types::FormFactor formFactor READ formFactor NOTIFY formFactorChanged)
+    Q_PROPERTY(Plasma::Types::Location location READ location NOTIFY locationChanged)
+    Q_PROPERTY(QString title READ title WRITE setTitle NOTIFY titleChanged FINAL)
+    Q_PROPERTY(QString icon READ icon WRITE setIcon NOTIFY iconChanged FINAL)
     Q_PROPERTY(bool busy READ isBusy WRITE setBusy NOTIFY busyChanged FINAL)
 
-    /**
-     * True when the user is configuring, for instance when the configuration dialog is open.
-     */
-    Q_PROPERTY(bool userConfiguring READ isUserConfiguring NOTIFY userConfiguringChanged)
-
-    /**
-     * How the applet wants its background to be drawn. The containment may chose to ignore this hint.
-     */
-    Q_PROPERTY(Plasma::Types::BackgroundHints backgroundHints WRITE setBackgroundHints READ backgroundHints NOTIFY backgroundHintsChanged FINAL)
-
-    /**
-     * The containment (and/or the user) may decide to use another kind of background instead (if supported by the applet)
-     */
-    Q_PROPERTY(Plasma::Types::BackgroundHints userBackgroundHints WRITE setUserBackgroundHints READ userBackgroundHints NOTIFY userBackgroundHintsChanged FINAL)
-
-    /**
-     * The effective background hints the applet has, internally decided how to mix with userBackgroundHints
-     */
-    Q_PROPERTY(Plasma::Types::BackgroundHints effectiveBackgroundHints READ effectiveBackgroundHints NOTIFY effectiveBackgroundHintsChanged FINAL)
-
-    // TODO KF6: activity, screen, screenGeometry, availableScreenRect, availableScreenRegion: should we instead make the containment accessible from qml
-    // plasmoids and ask from there?
-
-    /**
-     * A KConfigPropertyMap instance that represents the configuration
-     * which is usable from QML to read and write settings like any JavaScript Object
-     */
-    Q_PROPERTY(KConfigPropertyMap *configuration READ configuration CONSTANT FINAL)
-
-    /**
-     * The global shortcut to activate the plasmoid
-     *
-     * This is typically only used by the default configuration module
-     *
-     */
-    Q_PROPERTY(QKeySequence globalShortcut READ globalShortcut WRITE setGlobalShortcut RESET setGlobalShortcut NOTIFY globalShortcutChanged)
-
-    /**
-     * If true the applet requires manual configuration from the user
-     * TODO KF6: having just a reson property and required would be string not empty? Uglier from c++ pov but more straight forward from qml pov
-     */
-    Q_PROPERTY(bool configurationRequired READ configurationRequired WRITE setConfigurationRequired NOTIFY configurationRequiredChanged)
-
-    /**
-     * The hints that the applet gives to its constraint,
-     * such as asking to fill all the available space ignoring margins.
-     */
-    Q_PROPERTY(Applet::ConstraintHints constraintHints READ constraintHints WRITE setConstraintHints NOTIFY constraintHintsChanged FINAL)
-
-    /**
-     * The metadata of the applet.
-     */
-    Q_PROPERTY(KPluginMetaData metaData READ pluginMetaData CONSTANT)
-
-    /**
-     * The Containment managing this applet
-     */
-    Q_PROPERTY(Plasma::Containment *containment READ containment NOTIFY containmentChanged)
-
-    /**
-     * Actions to be added in the plasmoid context menu. To instantiate QActions in a declarative way,
-     * PlasmaCore.Action {} can be used
-     */
-    Q_PROPERTY(QQmlListProperty<QAction> contextualActions READ qmlContextualActions NOTIFY contextualActionsChanged)
-
-    /**
-     * True if this applet is a Containment and is acting as one, such as a desktop or a panel
-     */
-    Q_PROPERTY(bool isContainment READ isContainment CONSTANT)
-
-    /**
-     * Plugin name for the applet
-     */
-    Q_PROPERTY(QString pluginName READ pluginName CONSTANT FINAL)
-
 public:
+//CONSTRUCTORS
     /**
-     * The Constraint enumeration lists the various constraints that Plasma
-     * objects have managed for them and which they may wish to react to,
-     * for instance in Applet::constraintsUpdated
+     * @param parent the QObject this applet is parented to
+     * @param serviceId the name of the .desktop file containing the
+     *      information about the widget
+     * @param appletId a unique id used to differentiate between multiple
+     *      instances of the same Applet type
      */
-    enum Constraint {
-        NoConstraint = 0, /**< No constraint; never passed in to Applet::constraintsEvent on its own */
-        FormFactorConstraint = 1, /**< The FormFactor for an object */
-        LocationConstraint = 2, /**< The Location of an object */
-        ScreenConstraint = 4, /**< Which screen an object is on */
-        ImmutableConstraint = 8, /**< the immutability (locked) nature of the applet changed  */
-        StartupCompletedConstraint = 16, /**< application startup has completed */
-        UiReadyConstraint = 32,
-        /**< The ui has been completely loaded */ // (FIXME: merged with StartupCompletedConstraint?)
-        AllConstraints = FormFactorConstraint | LocationConstraint | ScreenConstraint | ImmutableConstraint,
-    };
-    Q_ENUM(Constraint)
-    Q_DECLARE_FLAGS(Constraints, Constraint)
+    explicit Applet(QObject *parent = nullptr, const QString &serviceId = QString(), uint appletId = 0);
 
     /**
-     * This enumeration lists the various hints that an applet can pass to its
-     * constraint regarding the way that it is represented
-     */
-    enum ConstraintHint {
-        NoHint = 0,
-        CanFillArea = 1,
-        /**< The CompactRepresentation can fill the area and ignore constraint margins*/ // (TODO: KF6 CanFillArea -> CompactRepresentationFillArea)
-        MarginAreasSeparator = CanFillArea | 2, /**< The applet acts as a separator between the standard and slim panel margin areas*/
-    };
-    Q_DECLARE_FLAGS(ConstraintHints, ConstraintHint)
-    Q_FLAG(ConstraintHints)
-
-    // CONSTRUCTORS
-
-    /**
-     * This constructor can be used with the KCoreAddons plugin loading system.
-     * The argument list is expected to have contain the KPackage of the applet,
-     * the meta data file path (for compatibility) and an applet ID which must be a base 10 number.
+     * @param parent the QObject this applet is parented to
+     * @param info the plugin information object for this Applet
+     * @param appletId a unique id used to differentiate between multiple
+     *      instances of the same Applet type
+     * @since 4.6
      *
-     * @param parent a QObject parent; you probably want to pass in 0
-     * @param data, KPluginMetaData used to create this plugin
-     * @param args a list of strings containing the applet id
-     * @Since 5.86
+     * @deprecated prefer using KPluginMetaData
      */
-    Applet(QObject *parentObject, const KPluginMetaData &data, const QVariantList &args);
+    PLASMA_DEPRECATED explicit Applet(const KPluginInfo &info, QObject *parent = nullptr, uint appletId = 0);
 
-    ~Applet() override;
+    /**
+     * @param parent the QObject this applet is parented to
+     * @param metadata the plugin information object for this Applet
+     * @param appletId a unique id used to differentiate between multiple
+     *      instances of the same Applet type
+     * @since 5.27
+     */
+    explicit Applet(const KPluginMetaData &metadata, QObject *parent = nullptr, uint appletId = 0);
 
-    // BOOKKEEPING
+    ~Applet();
+
+//BOOKKEEPING
     /**
      * @return the id of this applet
      */
     uint id() const;
 
-    /**
-     * @return the arguments this applet was started with.
-     * Some applets support arguments, for instance the notes applet supports to be
-     * instantiated with a given text already passed as paramenter
-     */
-    QVariantList startupArguments() const;
-
     /**
      * @return The type of immutability of this applet
      */
     Types::ImmutabilityType immutability() const;
 
-    /**
-     * @return true if immutability() is not Types::Mutable
-     */
-    bool immutable() const;
-
     /**
      * If for some reason, the applet fails to get up on its feet (the
      * library couldn't be loaded, necessary hardware support wasn't found,
@@ -322,20 +170,14 @@ public:
      */
     Types::Location location() const;
 
+//CONFIGURATION
     /**
-     * @return Display hints that come from the containment that suggest the applet how to look and behave.
-     * @since 5.77
-     */
-    Types::ContainmentDisplayHints containmentDisplayHints() const;
-
-    // CONFIGURATION
-    /**
-     * Returns the KConfigGroup to access the applets configuration.
-     *
-     * This config object will write to an instance
-     * specific config file named \<appletname\>\<instanceid\>rc
-     * in the Plasma appdata directory.
-     **/
+    * Returns the KConfigGroup to access the applets configuration.
+    *
+    * This config object will write to an instance
+    * specific config file named \<appletname\>\<instanceid\>rc
+    * in the Plasma appdata directory.
+    **/
     KConfigGroup config() const;
 
     /**
@@ -355,17 +197,11 @@ public:
      **/
     KConfigLoader *configScheme() const;
 
-    /**
-     * @return a KConfigPropertyMap instance that represents the configuration
-     * which is usable from QML to read and write settings like any JavaScript Object
-     */
-    KConfigPropertyMap *configuration();
-
     /**
      * Saves state information about this applet that will
      * be accessed when next instantiated in the restore(KConfigGroup&) method.
      *
-     * This method does not need to be reimplemented by Applet
+     * This method does not need to be reimplmented by Applet
      * subclasses, but can be useful for Applet specializations
      * (such as Containment) to do so.
      *
@@ -377,25 +213,12 @@ public:
      * Restores state information about this applet saved previously
      * in save(KConfigGroup&).
      *
-     * This method does not need to be reimplemented by Applet
+     * This method does not need to be reimplmented by Applet
      * subclasses, but can be useful for Applet specializations
      * (such as Containment) to do so.
      **/
     virtual void restore(KConfigGroup &group);
 
-    /**
-     * When the applet needs to be configured before being usable, this
-     * method can be called to show a standard interface prompting the user
-     * to configure the applet
-     *
-     * @param needsConfiguring true if the applet needs to be configured,
-     *                         or false if it doesn't
-     * @param reason a translated message for the user explaining that the
-     *               applet needs configuring; this should note what needs
-     *               to be configured
-     */
-    void setConfigurationRequired(bool needsConfiguring, const QString &reason = QString());
-
     /**
      * @return true if the applet currently needs to be configured,
      *         otherwise, false
@@ -412,21 +235,6 @@ public:
      */
     QString configurationRequiredReason() const;
 
-    /**
-     * Sets the constraint hits which give a more granular control over sizing in
-     * constrained layouts such as panels
-     *
-     * @param constraintHints such as CanFillArea or MarginAreasSeparator,
-     *                        they can be in bitwise OR
-     */
-    void setConstraintHints(ConstraintHints constraintHints);
-
-    /**
-     * @return The constraint hints such as CanFillArea or MarginAreasSeparator,
-     *         they can be in bitwise OR
-     */
-    ConstraintHints constraintHints() const;
-
     /**
      * @return true when the configuration interface is being shown
      * @since 4.5
@@ -439,7 +247,27 @@ public:
      */
     void setUserConfiguring(bool configuring);
 
-    // UTILS
+//UTILS
+#ifndef PLASMA_NO_DEPRECATED
+    /**
+     * Accessor for the associated Package object if any.
+     * Generally, only Plasmoids come in a Package.
+     *
+     * @deprecated use kPackage() instead
+     * @return the Package object, or an invalid one if none
+     **/
+    PLASMA_DEPRECATED Package package() const;
+#endif
+
+    /**
+     * Accessor for the associated Package object if any.
+     * Generally, only Plasmoids come in a Package.
+     *
+     * @return the Package object, or an invalid one if none
+     * @since 5.6
+     **/
+    KPackage::Package kPackage() const;
+
     /**
      * Called when any of the geometry constraints have been updated.
      * This method calls constraintsEvent, which may be reimplemented,
@@ -447,9 +275,16 @@ public:
      *
      * @param constraints the type of constraints that were updated
      */
-    void updateConstraints(Constraints constraints = AllConstraints);
+    void updateConstraints(Plasma::Types::Constraints constraints = Plasma::Types::AllConstraints);
 
-    // METADATA
+//METADATA
+    /**
+     * @return metadata information about this plugin
+     * @see KPluginInfo, pluginMetaData
+     * @since 5.0
+     * @deprecated use pluginMetaData instead
+     */
+    PLASMA_DEPRECATED KPluginInfo pluginInfo() const;
 
     /**
      * @return metadata information about this plugin
@@ -458,11 +293,6 @@ public:
      */
     KPluginMetaData pluginMetaData() const;
 
-    /**
-     * @return the plugin name form KPluginMetaData
-     */
-    QString pluginName() const;
-
     /**
      * Returns the user-visible title for the applet, as specified in the
      * Name field of the .desktop file. Can be changed with @see setTitle
@@ -481,6 +311,26 @@ public:
      */
     void setTitle(const QString &title);
 
+    /**
+     * Attempts to load an applet from a package
+     *
+     * Returns a pointer to the applet if successful.
+     * The caller takes responsibility for the applet, including
+     * deleting it when no longer needed.
+     * If you instance a plasmoid with this deprecated API, the
+     * automated default setup scripts won't be executed for that plasmoid
+     *
+     * @param path the path to the package
+     * @param appletId unique ID to assign the applet, or zero to have one
+     *        assigned automatically.
+     * @return a pointer to the loaded applet, or 0 on load failure
+     * @since 4.3
+     *
+     * @deprecated use Containment::createApplet() instead, you are not
+     * supposed to have applets without containments
+     **/
+    PLASMA_DEPRECATED static Applet *loadPlasmoid(const QString &path, uint appletId = 0);
+
     /**
      * @returns The icon name related to this applet
      * By default is the one in the plasmoid desktop file
@@ -510,40 +360,7 @@ public:
      */
     void setBusy(bool busy);
 
-    /**
-     * How the applet wants its background to be drawn. The containment may chose to ignore this hint.
-     * @since 5.65
-     */
-    Plasma::Types::BackgroundHints backgroundHints() const;
-
-    /**
-     * Sets the applet background hints. Only Applet implementations should write this property
-     * @since 5.65
-     */
-    void setBackgroundHints(Plasma::Types::BackgroundHints hint);
-
-    /**
-     * The containment (and/or the user) may decide to use another kind of background instead if supported by the applet.
-     * In order for an applet to support user configuration of the
-     * background, it needs to have the Plasma::Types::ConfigurableBackground flag set in its backgroundHints
-     * @since 5.65
-     */
-    Plasma::Types::BackgroundHints userBackgroundHints() const;
-
-    /**
-     * Sets the hints the user wished the background style for the applet to be.
-     * @since 5.65
-     */
-    void setUserBackgroundHints(Plasma::Types::BackgroundHints hint);
-
-    /**
-     * The effective background hints the applet will have: it will follow userBackgroundHints only if backgroundHints has the
-     * Plasma::Types::ConfigurableBackground flag set
-     * @since 5.65
-     */
-    Plasma::Types::BackgroundHints effectiveBackgroundHints() const;
-
-    // ACTIONS
+//ACTIONS
     /**
      * Returns a list of context-related QAction instances.
      *
@@ -555,66 +372,71 @@ public:
      **/
     virtual QList<QAction *> contextualActions();
 
-    QML_LIST_PROPERTY_ASSIGN_BEHAVIOR_REPLACE
-    QQmlListProperty<QAction> qmlContextualActions();
-
     /**
-     * Add a new internal action. if an internal action with the same name already exists, it
-     * will be replaced with this new one.
-     * Those are usually actions defined by the system, such as "configure" and "remove"
-     *
-     * @param name The unique name for the action
-     * @param action The new QAction to be added
+     * Returns the collection of actions for this Applet
      */
-    Q_INVOKABLE void setInternalAction(const QString &name, QAction *action);
-
-    /**
-     * @returns the internal action with the given name if available
-     * @param name the unique name of the action we want
-     */
-    Q_INVOKABLE QAction *internalAction(const QString &name) const;
-
-    /**
-     * Removes an action from the internal actions
-     * @param name the action to be removed
-     */
-    Q_INVOKABLE void removeInternalAction(const QString &name);
-
-    /**
-     * @returns All the internal actions such as configure, remove, alternatives etc
-     */
-    QList<QAction *> internalActions() const;
+    KActionCollection *actions() const;
 
     /**
      * Sets the global shortcut to associate with this widget.
      */
-    void setGlobalShortcut(const QKeySequence &shortcut = QKeySequence());
+    void setGlobalShortcut(const QKeySequence &shortcut);
 
     /**
-     * @return the global shortcut associated with this widget, or
+     * @return the global shortcut associated with this wiget, or
      * an empty shortcut if no global shortcut is associated.
      */
     QKeySequence globalShortcut() const;
 
+// ASSOCIATED APPLICATION
     /**
-     * Sets whether or not this applet provides a user interface for
-     * configuring the applet.
+     * Sets an application associated to this applet, that will be
+     * regarded as a full view of what is represented in the applet
      *
-     * It defaults to false, and if true is passed in you should
-     * also reimplement createConfigurationInterface()
-     *
-     * @param hasInterface whether or not there is a user interface available
-     **/
-    void setHasConfigurationInterface(bool hasInterface);
+     * @param string the name of the application. it can be
+     *      \li a name understood by KService::serviceByDesktopName
+     *        (e.g. "konqueror")
+     *      \li a command in $PATH
+     *      \li or an absolute path to an executable
+     * @since 4.4
+     */
+    void setAssociatedApplication(const QString &string);
 
-    // Completely UI-specific, remove or move to scriptengine
+    /**
+     * Sets a list of urls associated to this application,
+     * they will be used as parameters for the associated application
+     * @see setAssociatedApplication()
+     *
+     * @param urls
+     */
+    void setAssociatedApplicationUrls(const QList<QUrl> &urls);
+
+    /**
+     * @return the application associated to this applet
+     * @since 4.4
+     */
+    QString associatedApplication() const;
+
+    /**
+     * @return the urls associated to this applet
+     * @since 4.4
+     */
+    QList<QUrl> associatedApplicationUrls() const;
+
+    /**
+     * @return true if the applet has a valid associated application or urls
+     * @since 4.4
+     */
+    bool hasValidAssociatedApplication() const;
+
+//Completely UI-specific, remove or move to scriptengine
     /**
      * @return true if this plasmoid provides a GUI configuration
      **/
     bool hasConfigurationInterface() const;
 
 Q_SIGNALS:
-    // BOOKKEEPING
+//BOOKEEPING
     /**
      * Emitted when the immutability changes
      * @since 4.4
@@ -652,30 +474,7 @@ Q_SIGNALS:
      */
     void busyChanged(bool busy);
 
-    /**
-     * Emitted when the background hints have changed
-     * @since 5.65
-     */
-    void backgroundHintsChanged();
-
-    /**
-     * Emitted when the user background hints have changed
-     * @since 5.65
-     */
-    void userBackgroundHintsChanged();
-
-    /**
-     * Emitted when the effective background hints have changed
-     * @since 5.65
-     */
-    void effectiveBackgroundHintsChanged();
-
-    /**
-     * Emitted when the global shortcut to activate this applet has chanaged
-     */
-    void globalShortcutChanged(const QKeySequence &sequence);
-
-    // CONFIGURATION
+//CONFIGURATION
     /**
      * Emitted when an applet has changed values in its configuration
      * and wishes for them to be saved at the next save point. As this implies
@@ -691,7 +490,7 @@ Q_SIGNALS:
      */
     void userConfiguringChanged(bool configuring);
 
-    // ACTIONS
+//ACTIONS
     /**
      * Emitted just before the contextual actions are about to show
      * For instance just before the context menu containing the actions
@@ -701,11 +500,11 @@ Q_SIGNALS:
 
     /**
      * Emitted when activation is requested due to, for example, a global
-     * keyboard shortcut. By default the widget is given focus.
+     * keyboard shortcut. By default the wiget is given focus.
      */
     void activated();
 
-    // TODO: fix usage in containment, port to QObject::destroyed
+//TODO: fix usage in containment, port to QObject::destroyed
     /**
      * Emitted when the applet is deleted
      */
@@ -721,8 +520,6 @@ Q_SIGNALS:
      */
     void locationChanged(Plasma::Types::Location location);
 
-    void containmentDisplayHintsChanged(Plasma::Types::ContainmentDisplayHints hints);
-
     /**
      * Emitted when setConfigurationRequired was called
      * @see setConfigurationRequired
@@ -730,30 +527,8 @@ Q_SIGNALS:
      */
     void configurationRequiredChanged(bool needsConfig, const QString &reason);
 
-    /**
-     * Emitted when the constraint hints changed
-     * @see setConstraintHints
-     */
-    void constraintHintsChanged(ConstraintHints constraintHints);
-
-    /**
-     * Emitted when the containment changes
-     */
-    void containmentChanged(Plasma::Containment *containment);
-
-    /**
-     * Emitted when the list of contextual actions has changed
-     */
-    void contextualActionsChanged(const QList<QAction *> &actions);
-
-    /**
-     * Emitted when the list of internal actions has changed
-     */
-    void internalActionsChanged(const QList<QAction *> &actions);
-
-    // TODO KF6 keep as Q_SLOT only stuff that needsto be manually invokable from qml
 public Q_SLOTS:
-    // BOOKKEEPING
+//BOOKKEEPING
     /**
      * Call this method when the applet fails to launch properly. An
      * optional reason can be provided.
@@ -789,14 +564,13 @@ public Q_SLOTS:
      */
     void setStatus(const Types::ItemStatus stat);
 
-    // CONFIGURATION
+//CONFIGURATION
     /**
      * Called when applet configuration values have changed.
      */
-    // TODO KF6: make it not a slot anymore and protected
     virtual void configChanged();
 
-    // UTILS
+//UTILS
     /**
      * Sends all pending constraints updates to the applet. Will usually
      * be called automatically, but can also be called manually if needed.
@@ -815,8 +589,32 @@ public Q_SLOTS:
      **/
     virtual void init();
 
+//ASSOCIATED APPLICATION
+    /**
+    * Open the application associated to this applet, if it's not set
+    * but some urls are, open those urls with the proper application
+    * for their mimetype
+    * @see setAssociatedApplication()
+    * @see setAssociatedApplicationUrls()
+    * @since 4.4
+    */
+    void runAssociatedApplication();
+
 protected:
-    // CONFIGURATION
+//CONSTRUCTORS
+    /**
+     * This constructor is to be used with the plugin loading systems
+     * found in KPluginInfo and KService. The argument list is expected
+     * to have two elements: the KService service ID for the desktop entry
+     * and an applet ID which must be a base 10 number.
+     *
+     * @param parent a QObject parent; you probably want to pass in 0
+     * @param args a list of strings containing two entries: the service id
+     *      and the applet id
+     */
+    Applet(QObject *parent, const QVariantList &args);
+
+//CONFIGURATION
     /**
      * When called, the Applet should write any information needed as part
      * of the Applet's running state to the configuration object in config()
@@ -828,7 +626,31 @@ protected:
      **/
     virtual void saveState(KConfigGroup &config) const;
 
-    // UTILS
+    /**
+     * Sets whether or not this applet provides a user interface for
+     * configuring the applet.
+     *
+     * It defaults to false, and if true is passed in you should
+     * also reimplement createConfigurationInterface()
+     *
+     * @param hasInterface whether or not there is a user interface available
+     **/
+    void setHasConfigurationInterface(bool hasInterface);
+
+    /**
+     * When the applet needs to be configured before being usable, this
+     * method can be called to show a standard interface prompting the user
+     * to configure the applet
+     *
+     * @param needsConfiguring true if the applet needs to be configured,
+     *                         or false if it doesn't
+     * @param reason a translated message for the user explaining that the
+     *               applet needs configuring; this should note what needs
+     *               to be configured
+     */
+    void setConfigurationRequired(bool needsConfiguring, const QString &reason = QString());
+
+//UTILS
     /**
      * Called when any of the constraints for the applet have been updated. These constraints
      * range from notifying when the applet has officially "started up" to when geometry changes
@@ -847,17 +669,15 @@ protected:
      * @param constraints the type of constraints that were updated
      * @property constraint
      */
-    virtual void constraintsEvent(Constraints constraints);
+    virtual void constraintsEvent(Plasma::Types::Constraints constraints);
 
-    // TODO: timerEvent should go into AppletPrivate
+//TODO: timerEvent should go into AppletPrivate
     /**
      * Reimplemented from QObject
      */
-    void timerEvent(QTimerEvent *event) override;
+    void timerEvent(QTimerEvent *event) Q_DECL_OVERRIDE;
 
 private:
-    QString filePath(const QByteArray &key, const QString &filename = QString()) const;
-    KPackage::Package kPackage() const;
     /**
      * @internal This constructor is to be used with the Package loading system.
      *
@@ -868,16 +688,16 @@ private:
      */
     Applet(const QString &packagePath, uint appletId);
 
-    // TODO KF6: drop Q_PRIVATE_SLOT
     Q_PRIVATE_SLOT(d, void cleanUpAndDelete())
     Q_PRIVATE_SLOT(d, void askDestroy())
+    Q_PRIVATE_SLOT(d, void updateShortcuts())
     Q_PRIVATE_SLOT(d, void globalShortcutChanged())
     Q_PRIVATE_SLOT(d, void propagateConfigChanged())
     Q_PRIVATE_SLOT(d, void requestConfiguration())
 
     AppletPrivate *const d;
 
-    // Corona needs to access setLaunchErrorMessage and init
+    //Corona needs to access setLaunchErrorMessage and init
     friend class Corona;
     friend class CoronaPrivate;
     friend class Containment;
@@ -887,18 +707,20 @@ private:
     friend class AccessAppletJobPrivate;
     friend class GraphicsViewAppletPrivate;
     friend class PluginLoader;
-    friend class SvgPrivate;
-    friend class PlasmaQuick::AppletQuickItem;
-    friend class PlasmaQuick::ConfigModel;
-    friend class PlasmaQuick::ConfigModelPrivate;
-    friend class PlasmaQuick::ConfigViewPrivate;
-    friend class PlasmaQuick::ConfigView;
-    friend DeclarativeAppletScript;
+    friend class AssociatedApplicationManager;
 };
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(Applet::Constraints)
-Q_DECLARE_OPERATORS_FOR_FLAGS(Applet::ConstraintHints)
-
 } // Plasma namespace
 
+/**
+ * Register an applet when it is contained in a loadable module
+ */
+#define K_EXPORT_PLASMA_APPLET(libname, classname) \
+    K_PLUGIN_FACTORY(factory, registerPlugin<classname>();) \
+    K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+
+#define K_EXPORT_PLASMA_APPLET_WITH_JSON(libname, classname, jsonFile) \
+    K_PLUGIN_FACTORY_WITH_JSON(factory, jsonFile, registerPlugin<classname>();) \
+    K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+
 #endif // multiple inclusion guard
