@@ -1,42 +1,62 @@
 /*
-    SPDX-FileCopyrightText: 2007 Matt Broadstone <mbroadst@gmail.com>
-    SPDX-FileCopyrightText: 2007-2011 Aaron Seigo <aseigo@kde.org>
-    SPDX-FileCopyrightText: 2007 Riccardo Iaconelli <riccardo@kde.org>
-    SPDX-FileCopyrightText: 2009 Chani Armitage <chani@kde.org>
-
-    SPDX-License-Identifier: LGPL-2.0-or-later
-*/
+ *   Copyright 2007 Matt Broadstone <mbroadst@gmail.com>
+ *   Copyright 2007-2011 Aaron Seigo <aseigo@kde.org>
+ *   Copyright 2007 Riccardo Iaconelli <riccardo@kde.org>
+ *   Copyright (c) 2009 Chani Armitage <chani@kde.org>
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU Library General Public License as
+ *   published by the Free Software Foundation; either version 2, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details
+ *
+ *   You should have received a copy of the GNU Library General Public
+ *   License along with this program; if not, write to the
+ *   Free Software Foundation, Inc.,
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
 
 #include "corona.h"
 #include "private/corona_p.h"
 
-#include <QDebug>
 #include <QGuiApplication>
-#include <QJSEngine>
 #include <QMimeData>
 #include <QPainter>
-#include <QScreen>
 #include <QTimer>
-
-#include <KLocalizedString>
+#include <QScreen>
+#include <QScreen>
 
 #include <cmath>
 
+#include <QDebug>
+#include <klocalizedstring.h>
+#include <kwindowsystem.h>
+
 #include "containment.h"
-#include "debug_p.h"
 #include "pluginloader.h"
+#include "packagestructure.h"
 #include "private/applet_p.h"
 #include "private/containment_p.h"
+#include "private/package_p.h"
 #include "private/timetracker.h"
+#include "debug_p.h"
 
 using namespace Plasma;
 
 namespace Plasma
 {
+
 Corona::Corona(QObject *parent)
-    : QObject(parent)
-    , d(new CoronaPrivate(this))
+    : QObject(parent),
+      d(new CoronaPrivate(this))
 {
+#ifndef NDEBUG
+    // qCDebug(LOG_PLASMA) << "!!{} STARTUP TIME" << QTime().msecsTo(QTime::currentTime()) << "Corona ctor start";
+#endif
     d->init();
 
 #ifndef NDEBUG
@@ -48,12 +68,25 @@ Corona::Corona(QObject *parent)
 
 Corona::~Corona()
 {
-    KConfigGroup trans(KSharedConfig::openConfig(), QStringLiteral("PlasmaTransientsConfig"));
+    KConfigGroup trans(KSharedConfig::openConfig(), "PlasmaTransientsConfig");
     trans.deleteGroup();
 
     delete d;
 }
 
+Plasma::Package Corona::package() const
+{
+    Package p;
+    p.d->internalPackage = new KPackage::Package(d->package);
+    return p;
+}
+
+void Corona::setPackage(const Plasma::Package &package)
+{
+    setKPackage(*package.d->internalPackage);
+    emit packageChanged(package);
+}
+
 KPackage::Package Corona::kPackage() const
 {
     return d->package;
@@ -62,7 +95,7 @@ KPackage::Package Corona::kPackage() const
 void Corona::setKPackage(const KPackage::Package &package)
 {
     d->package = package;
-    Q_EMIT kPackageChanged(package);
+    emit kPackageChanged(package);
 }
 
 void Corona::saveLayout(const QString &configName) const
@@ -80,34 +113,32 @@ void Corona::saveLayout(const QString &configName) const
 
 void Corona::exportLayout(KConfigGroup &config, QList<Containment *> containments)
 {
-    const auto groupList = config.groupList();
-    for (const QString &group : groupList) {
+    foreach (const QString &group, config.groupList()) {
         KConfigGroup cg(&config, group);
         cg.deleteGroup();
     }
 
-    // temporarily unlock so that removal works
+    //temporarily unlock so that removal works
     Types::ImmutabilityType oldImm = immutability();
     d->immutability = Types::Mutable;
 
-    KConfigGroup dest(&config, QStringLiteral("Containments"));
+    KConfigGroup dest(&config, "Containments");
     KConfigGroup dummy;
-    for (Plasma::Containment *c : std::as_const(containments)) {
+    foreach (Plasma::Containment *c, containments) {
         c->save(dummy);
         c->config().reparent(&dest);
 
-        // ensure the containment is unlocked
-        // this is done directly because we have to bypass any Types::SystemImmutable checks
+        //ensure the containment is unlocked
+        //this is done directly because we have to bypass any Types::SystemImmutable checks
         c->Applet::d->immutability = Types::Mutable;
-        const auto lstApplet = c->applets();
-        for (Applet *a : lstApplet) {
+        foreach (Applet *a, c->applets()) {
             a->d->immutability = Types::Mutable;
         }
 
         c->destroy();
     }
 
-    // restore immutability
+    //restore immutability
     d->immutability = oldImm;
 
     config.sync();
@@ -121,7 +152,7 @@ void Corona::requestConfigSync()
 
     // TODO: should we check into our immutability before doing this?
 
-    // NOTE: this is a pretty simplistic model: we simply save no more than CONFIG_SYNC_TIMEOUT
+    //NOTE: this is a pretty simplistic model: we simply save no more than CONFIG_SYNC_TIMEOUT
     //      after the first time this is called. not much of a heuristic for save points, but
     //      it should at least compress these activities a bit and provide a way for applet
     //      authors to ween themselves from the sync() disease. A more interesting/dynamic
@@ -140,7 +171,7 @@ void Corona::loadLayout(const QString &configName)
 {
     if (!configName.isEmpty() && configName != d->configName) {
         // if we have a new config name passed in, then use that as the config file for this Corona
-        d->config = nullptr;
+        d->config = 0;
         d->configName = configName;
     }
 
@@ -152,7 +183,7 @@ void Corona::loadLayout(const QString &configName)
         d->notifyContainmentsReady();
     }
 
-    KConfigGroup cg(config(), QStringLiteral("General"));
+    KConfigGroup cg(config(), "General");
     setImmutability((Plasma::Types::ImmutabilityType)cg.readEntry("immutability", (int)Plasma::Types::Mutable));
 }
 
@@ -161,69 +192,39 @@ QList<Plasma::Containment *> Corona::importLayout(const KConfigGroup &conf)
     return d->importLayout(conf, true);
 }
 
-Containment *Corona::containmentForScreen(int screen, const QString &activity, const QString &defaultPluginIfNonExistent, const QVariantList &defaultArgs)
+Containment *Corona::containmentForScreen(int screen) const
 {
-    Containment *containment = nullptr;
-
-    for (Containment *cont : std::as_const(d->containments)) {
-        if (cont->lastScreen() == screen //
-            && ((cont->activity().isEmpty() || activity.isEmpty()) || cont->activity() == activity)
-            && (cont->containmentType() == Plasma::Containment::Type::Desktop //
-                || cont->containmentType() == Plasma::Containment::Type::Custom)) {
-            containment = cont;
+    foreach (Containment *containment, d->containments) {
+        if (containment->screen() == screen &&
+                (containment->containmentType() == Plasma::Types::DesktopContainment ||
+                 containment->containmentType() == Plasma::Types::CustomContainment)) {
+            return containment;
         }
     }
 
+    return 0;
+}
+
+Containment *Corona::containmentForScreen(int screen,
+                                          const QString &defaultPluginIfNonExistent, const QVariantList &defaultArgs)
+{
+    Containment *containment = containmentForScreen(screen);
     if (!containment && !defaultPluginIfNonExistent.isEmpty()) {
         // screen requests are allowed to bypass immutability
         if (screen >= 0) {
             Plasma::Types::ImmutabilityType imm = d->immutability;
             d->immutability = Types::Mutable;
-            containment = d->addContainment(defaultPluginIfNonExistent, defaultArgs, 0, screen, false);
-
+            containment = d->addContainment(defaultPluginIfNonExistent, defaultArgs, 0, false);
+            if (containment) {
+               // containment->setScreen(screen);
+            }
             d->immutability = imm;
         }
     }
 
-    if (containment) {
-        containment->setActivity(activity);
-    }
     return containment;
 }
 
-QList<Containment *> Corona::containmentsForActivity(const QString &activity)
-{
-    QList<Containment *> conts;
-
-    if (activity.isEmpty()) {
-        return conts;
-    }
-
-    std::copy_if(d->containments.begin(), d->containments.end(), std::back_inserter(conts), [activity](Containment *cont) {
-        return cont->activity() == activity
-            && (cont->containmentType() == Plasma::Containment::Type::Desktop || cont->containmentType() == Plasma::Containment::Type::Custom);
-    });
-
-    return conts;
-}
-
-QList<Containment *> Corona::containmentsForScreen(int screen)
-{
-    QList<Containment *> conts;
-
-    if (screen < 0) {
-        return conts;
-    }
-
-    std::copy_if(d->containments.begin(), d->containments.end(), std::back_inserter(conts), [screen](Containment *cont) {
-        return cont->lastScreen() == screen
-            && (cont->containmentType() == Plasma::Containment::Type::Desktop //
-                || cont->containmentType() == Plasma::Containment::Type::Custom);
-    });
-
-    return conts;
-}
-
 QList<Containment *> Corona::containments() const
 {
     return d->containments;
@@ -246,19 +247,19 @@ KSharedConfigPtr Corona::config() const
 Containment *Corona::createContainment(const QString &name, const QVariantList &args)
 {
     if (d->immutability == Types::Mutable || args.contains(QVariant::fromValue(QStringLiteral("org.kde.plasma:force-create")))) {
-        return d->addContainment(name, args, 0, -1, false);
+        return d->addContainment(name, args, 0);
     }
 
-    return nullptr;
+    return 0;
 }
 
 Containment *Corona::createContainmentDelayed(const QString &name, const QVariantList &args)
 {
     if (d->immutability == Types::Mutable) {
-        return d->addContainment(name, args, 0, -1, true);
+        return d->addContainment(name, args, 0, true);
     }
 
-    return nullptr;
+    return 0;
 }
 
 int Corona::screenForContainment(const Containment *) const
@@ -283,7 +284,7 @@ QRect Corona::availableScreenRect(int id) const
 
 void Corona::loadDefaultLayout()
 {
-    // Default implementation does nothing
+    //Default implementation does nothing
 }
 
 Types::ImmutabilityType Corona::immutability() const
@@ -302,11 +303,11 @@ void Corona::setImmutability(const Types::ImmutabilityType immutable)
 #endif
     d->immutability = immutable;
     d->updateContainmentImmutability();
-    // tell non-containments that might care (like plasmaapp or a custom corona)
-    Q_EMIT immutabilityChanged(immutable);
+    //tell non-containments that might care (like plasmaapp or a custom corona)
+    emit immutabilityChanged(immutable);
 
-    // update our actions
-    QAction *action = d->actions.value(QStringLiteral("lock widgets"));
+    //update our actions
+    QAction *action = d->actions.action(QStringLiteral("lock widgets"));
     if (action) {
         if (d->immutability == Types::SystemImmutable) {
             action->setEnabled(false);
@@ -320,80 +321,25 @@ void Corona::setImmutability(const Types::ImmutabilityType immutable)
         }
     }
 
-    action = d->actions.value(QStringLiteral("edit mode"));
-    if (action) {
-        switch (d->immutability) {
-        case Types::UserImmutable:
-            action->setEnabled(false);
-            action->setVisible(true);
-            break;
-        case Types::SystemImmutable:
-            action->setEnabled(false);
-            action->setVisible(false);
-            break;
-        case Types::Mutable:
-        default:
-            action->setEnabled(true);
-            action->setVisible(true);
-            break;
-        }
-    }
-
     if (d->immutability != Types::SystemImmutable) {
-        KConfigGroup cg(config(), QStringLiteral("General"));
+        KConfigGroup cg(config(), "General");
 
         // we call the dptr member directly for locked since isImmutable()
         // also checks kiosk and parent containers
         cg.writeEntry("immutability", (int)d->immutability);
         requestConfigSync();
     }
-
-    if (d->immutability != Types::Mutable) {
-        setEditMode(false);
-    }
-}
-
-void Corona::setEditMode(bool edit)
-{
-    if (edit == d->editMode || (edit && d->immutability != Plasma::Types::Mutable)) {
-        return;
-    }
-
-    QAction *editAction = d->actions.value(QStringLiteral("edit mode"));
-    if (editAction) {
-        if (edit) {
-            editAction->setText(i18n("Exit Edit Mode"));
-        } else {
-            editAction->setText(i18n("Enter Edit Mode"));
-        }
-    }
-
-    if (!edit) {
-        requireConfigSync();
-    }
-
-    d->editMode = edit;
-    Q_EMIT editModeChanged(edit);
-}
-
-bool Corona::isEditMode() const
-{
-    return d->editMode;
 }
 
 QList<Plasma::Types::Location> Corona::freeEdges(int screen) const
 {
     QList<Plasma::Types::Location> freeEdges;
-    /* clang-format off */
-    freeEdges << Plasma::Types::TopEdge
-              << Plasma::Types::BottomEdge
-              << Plasma::Types::LeftEdge
-              << Plasma::Types::RightEdge;
-    /* clang-format on */
+    freeEdges << Plasma::Types::TopEdge << Plasma::Types::BottomEdge
+              << Plasma::Types::LeftEdge << Plasma::Types::RightEdge;
 
-    const auto containments = this->containments();
-    for (Containment *containment : containments) {
-        if (containment->screen() == screen && freeEdges.contains(containment->location())) {
+    foreach (Containment *containment, containments()) {
+        if (containment->screen() == screen &&
+                freeEdges.contains(containment->location())) {
             freeEdges.removeAll(containment->location());
         }
     }
@@ -401,49 +347,21 @@ QList<Plasma::Types::Location> Corona::freeEdges(int screen) const
     return freeEdges;
 }
 
-QAction *Corona::action(const QString &name) const
+KActionCollection *Corona::actions() const
 {
-    return d->actions.value(name);
-}
-
-void Corona::setAction(const QString &name, QAction *action)
-{
-    if (name.isEmpty()) {
-        return;
-    }
-    action->setObjectName(name);
-    QAction *oldAction = d->actions.value(name);
-    if (oldAction && QJSEngine::objectOwnership(oldAction) == QJSEngine::CppOwnership) {
-        delete oldAction;
-    }
-    connect(action, &QObject::destroyed, this, [this, name]() {
-        d->actions.remove(name);
-    });
-    d->actions[name] = action;
-}
-
-void Corona::removeAction(const QString &name)
-{
-    QAction *action = d->actions.value(name);
-    if (action && QJSEngine::objectOwnership(action) == QJSEngine::CppOwnership) {
-        delete action;
-    }
-    d->actions.remove(name);
-}
-
-QList<QAction *> Corona::actions() const
-{
-    return d->actions.values();
+    return &d->actions;
 }
 
 CoronaPrivate::CoronaPrivate(Corona *corona)
-    : q(corona)
-    , immutability(Types::Mutable)
-    , config(nullptr)
-    , configSyncTimer(new QTimer(corona))
-    , containmentsStarting(0)
+    : q(corona),
+      immutability(Types::Mutable),
+      config(0),
+      configSyncTimer(new QTimer(corona)),
+      actions(corona),
+      containmentsStarting(0)
 {
-    // TODO: make Package path configurable
+    //TODO: make Package path configurable
+    KConfigGroup config(KSharedConfig::openConfig(), "General");
 
     if (QCoreApplication::instance()) {
         configName = QCoreApplication::instance()->applicationName() + QStringLiteral("-appletsrc");
@@ -459,34 +377,26 @@ CoronaPrivate::~CoronaPrivate()
 
 void CoronaPrivate::init()
 {
-    desktopDefaultsConfig = KConfigGroup(KSharedConfig::openConfig(package.filePath("defaults")), QStringLiteral("Desktop"));
+    desktopDefaultsConfig = KConfigGroup(KSharedConfig::openConfig(package.filePath("defaults")), "Desktop");
 
     configSyncTimer->setSingleShot(true);
     QObject::connect(configSyncTimer, SIGNAL(timeout()), q, SLOT(syncConfig()));
 
-    QAction *lockAction = new QAction(q);
-    q->setAction(QStringLiteral("lock widgets"), lockAction);
+    //some common actions
+    actions.setConfigGroup(QStringLiteral("Shortcuts"));
+
+    QAction *lockAction = actions.add<QAction>(QStringLiteral("lock widgets"));
     QObject::connect(lockAction, SIGNAL(triggered(bool)), q, SLOT(toggleImmutability()));
     lockAction->setText(i18n("Lock Widgets"));
     lockAction->setAutoRepeat(true);
     lockAction->setIcon(QIcon::fromTheme(QStringLiteral("object-locked")));
+    lockAction->setData(Plasma::Types::ControlAction);
+    lockAction->setShortcut(QKeySequence(QStringLiteral("alt+d, l")));
     lockAction->setShortcutContext(Qt::ApplicationShortcut);
 
-    // fake containment/applet actions
-    auto containmentActions = AppletPrivate::defaultActions(q); // containment has to start with applet stuff
-    ContainmentPrivate::addDefaultActions(containmentActions, nullptr, q); // now it's really containment
-    actions.insert(containmentActions);
-
-    QAction *editAction = new QAction(q);
-    q->setAction(QStringLiteral("edit mode"), editAction);
-    QObject::connect(editAction, &QAction::triggered, q, [this]() {
-        q->setEditMode(!q->isEditMode());
-    });
-    editAction->setText(i18n("Enter Edit Mode"));
-    editAction->setAutoRepeat(true);
-    editAction->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
-    editAction->setShortcut(QKeySequence(QStringLiteral("alt+d, e")));
-    editAction->setShortcutContext(Qt::ApplicationShortcut);
+    //fake containment/applet actions
+    KActionCollection *containmentActions = AppletPrivate::defaultActions(q); //containment has to start with applet stuff
+    ContainmentPrivate::addDefaultActions(containmentActions); //now it's really containment
 }
 
 void CoronaPrivate::toggleImmutability()
@@ -500,8 +410,8 @@ void CoronaPrivate::toggleImmutability()
 
 void CoronaPrivate::saveLayout(KSharedConfigPtr cg) const
 {
-    KConfigGroup containmentsGroup(cg, QStringLiteral("Containments"));
-    for (const Containment *containment : containments) {
+    KConfigGroup containmentsGroup(cg, "Containments");
+    foreach (const Containment *containment, containments) {
         QString cid = QString::number(containment->id());
         KConfigGroup containmentConfig(&containmentsGroup, cid);
         containment->save(containmentConfig);
@@ -510,9 +420,9 @@ void CoronaPrivate::saveLayout(KSharedConfigPtr cg) const
 
 void CoronaPrivate::updateContainmentImmutability()
 {
-    for (Containment *c : std::as_const(containments)) {
+    foreach (Containment *c, containments) {
         // we need to tell each containment that immutability has been altered
-        c->updateConstraints(Applet::ImmutableConstraint);
+        c->updateConstraints(Types::ImmutableConstraint);
     }
 }
 
@@ -534,14 +444,14 @@ void CoronaPrivate::containmentDestroyed(QObject *obj)
 void CoronaPrivate::syncConfig()
 {
     q->config()->sync();
-    Q_EMIT q->configSynced();
+    emit q->configSynced();
 }
 
-Containment *CoronaPrivate::addContainment(const QString &name, const QVariantList &args, uint id, int lastScreen, bool delayedInit)
+Containment *CoronaPrivate::addContainment(const QString &name, const QVariantList &args, uint id, bool delayedInit)
 {
     QString pluginName = name;
-    Containment *containment = nullptr;
-    Applet *applet = nullptr;
+    Containment *containment = 0;
+    Applet *applet = 0;
 
     // qCDebug(LOG_PLASMA) << "Loading" << name << args << id;
 
@@ -574,12 +484,9 @@ Containment *CoronaPrivate::addContainment(const QString &name, const QVariantLi
             applet->init();
             delete applet;
         }
-        applet = containment = new Containment(q, KPluginMetaData(), QVariantList{QVariant(), id});
-        if (lastScreen >= 0) {
-            containment->d->lastScreen = lastScreen;
-        }
-        // if it's a dummy containment, just say its ui is ready, not blocking the corona
-        applet->updateConstraints(Applet::UiReadyConstraint);
+        applet = containment = new Containment(q, {}, id);
+        //if it's a dummy containment, just say its ui is ready, not blocking the corona
+        applet->updateConstraints(Plasma::Types::UiReadyConstraint);
 
         // we want to provide something and don't care about the failure to launch
         containment->setFormFactor(Plasma::Types::Planar);
@@ -588,33 +495,36 @@ Containment *CoronaPrivate::addContainment(const QString &name, const QVariantLi
     // if this is a new containment, we need to ensure that there are no stale
     // configuration data around
     if (id == 0) {
-        KConfigGroup conf(q->config(), QStringLiteral("Containments"));
+        KConfigGroup conf(q->config(), "Containments");
         conf = KConfigGroup(&conf, QString::number(containment->id()));
         conf.deleteGroup();
     }
 
-    // make sure the containments are sorted by id
-    auto position = std::lower_bound(containments.begin(), containments.end(), containment, [](Plasma::Containment *c1, Plasma::Containment *c2) {
+    //make sure the containments are sorted by id
+    auto position = std::lower_bound(containments.begin(), containments.end(), containment, [](Plasma::Containment *c1,  Plasma::Containment *c2) {
         return c1->id() < c2->id();
     });
     containments.insert(position, containment);
 
-    QObject::connect(containment, SIGNAL(destroyed(QObject *)), q, SLOT(containmentDestroyed(QObject *)));
-    QObject::connect(containment, &Applet::configNeedsSaving, q, &Corona::requestConfigSync);
-    QObject::connect(containment, &Containment::screenChanged, q, &Corona::screenOwnerChanged);
+    QObject::connect(containment, SIGNAL(destroyed(QObject*)),
+                     q, SLOT(containmentDestroyed(QObject*)));
+    QObject::connect(containment, SIGNAL(configNeedsSaving()),
+                     q, SLOT(requestConfigSync()));
+    QObject::connect(containment, SIGNAL(screenChanged(int)),
+                     q, SIGNAL(screenOwnerChanged(int)));
 
     if (!delayedInit) {
         containment->init();
         KConfigGroup cg = containment->config();
         containment->restore(cg);
-        containment->updateConstraints(Applet::StartupCompletedConstraint);
+        containment->updateConstraints(Plasma::Types::StartupCompletedConstraint);
         containment->save(cg);
         q->requestConfigSync();
         containment->flushPendingConstraintsEvents();
-        Q_EMIT q->containmentAdded(containment);
-        // if id = 0 a new containment has been created, not restored
+        emit q->containmentAdded(containment);
+        //if id = 0 a new containment has been created, not restored
         if (id == 0) {
-            Q_EMIT q->containmentCreated(containment);
+            emit q->containmentCreated(containment);
         }
     }
 
@@ -630,22 +540,19 @@ QList<Plasma::Containment *> CoronaPrivate::importLayout(const KConfigGroup &con
     QList<Plasma::Containment *> newContainments;
     QSet<uint> containmentsIds;
 
-    for (Containment *containment : std::as_const(containments)) {
+    foreach (Containment *containment, containments) {
         containmentsIds.insert(containment->id());
     }
 
-    KConfigGroup containmentsGroup(&conf, QStringLiteral("Containments"));
+    KConfigGroup containmentsGroup(&conf, "Containments");
     QStringList groups = containmentsGroup.groupList();
-    std::sort(groups.begin(), groups.end());
+    qSort(groups.begin(), groups.end());
 
-    for (const QString &group : std::as_const(groups)) {
+    foreach (const QString &group, groups) {
         KConfigGroup containmentConfig(&containmentsGroup, group);
 
         if (containmentConfig.entryMap().isEmpty()) {
             continue;
-        } else if (containmentConfig.readEntry(QStringLiteral("transient"), false)) {
-            containmentConfig.deleteGroup();
-            continue;
         }
 
         uint cid = group.toUInt();
@@ -656,19 +563,18 @@ QList<Plasma::Containment *> CoronaPrivate::importLayout(const KConfigGroup &con
         }
 
         if (mergeConfig) {
-            KConfigGroup realConf(q->config(), QStringLiteral("Containments"));
+            KConfigGroup realConf(q->config(), "Containments");
             realConf = KConfigGroup(&realConf, QString::number(cid));
             // in case something was there before us
             realConf.deleteGroup();
             containmentConfig.copyTo(&realConf);
         }
 
-        // qCDebug(LOG_PLASMA) << "got a containment in the config, trying to make a" << containmentConfig.readEntry("plugin", QString()) << "from" << group;
+        //qCDebug(LOG_PLASMA) << "got a containment in the config, trying to make a" << containmentConfig.readEntry("plugin", QString()) << "from" << group;
 #ifndef NDEBUG
-        // qCDebug(LOG_PLASMA) << "!!{} STARTUP TIME" << QTime().msecsTo(QTime::currentTime()) << "Adding Containment" << containmentConfig.readEntry("plugin",
-        // QString());
+        // qCDebug(LOG_PLASMA) << "!!{} STARTUP TIME" << QTime().msecsTo(QTime::currentTime()) << "Adding Containment" << containmentConfig.readEntry("plugin", QString());
 #endif
-        Containment *c = addContainment(containmentConfig.readEntry("plugin", QString()), QVariantList(), cid, -1);
+        Containment *c = addContainment(containmentConfig.readEntry("plugin", QString()), QVariantList(), cid);
         if (!c) {
             continue;
         }
@@ -691,17 +597,15 @@ QList<Plasma::Containment *> CoronaPrivate::importLayout(const KConfigGroup &con
 void CoronaPrivate::notifyContainmentsReady()
 {
     containmentsStarting = 0;
-    for (Containment *containment : std::as_const(containments)) {
+    foreach (Containment *containment, containments) {
         if (!containment->isUiReady() && containment->screen() >= 0) {
             ++containmentsStarting;
-            QObject::connect(containment, &Plasma::Containment::uiReadyChanged, q, [this](bool ready) {
-                containmentReady(ready);
-            });
+            QObject::connect(containment, &Plasma::Containment::uiReadyChanged, q, [this](bool ready) { containmentReady(ready); } );
         }
     }
 
     if (containmentsStarting <= 0) {
-        Q_EMIT q->startupCompleted();
+        emit q->startupCompleted();
     }
 }
 
@@ -712,7 +616,7 @@ void CoronaPrivate::containmentReady(bool ready)
     }
     --containmentsStarting;
     if (containmentsStarting <= 0) {
-        Q_EMIT q->startupCompleted();
+        emit q->startupCompleted();
     }
 }
 
