@@ -94,7 +94,7 @@ bool DeclarativeAppletScript::init()
 
 QString DeclarativeAppletScript::filePath(const QString &type, const QString &file) const
 {
-    return package().filePath(type.toLocal8Bit().constData(), file);
+    return applet()->kPackage().filePath(type.toLocal8Bit().constData(), file);
 }
 
 void DeclarativeAppletScript::constraintsEvent(Plasma::Types::Constraints constraints)
