@@ -31,7 +31,6 @@
 #include <QQmlExpression>
 #include <QQmlContext>
 #include <QQmlProperty>
-#include <QSignalMapper>
 
 #include <Plasma/PluginLoader>
 #include <kpackage/packageloader.h>
@@ -43,8 +42,7 @@ WallpaperInterface::WallpaperInterface(ContainmentInterface *parent)
       m_containmentInterface(parent),
       m_qmlObject(0),
       m_configuration(0),
-      m_configLoader(0),
-      m_actionSignals(0)
+      m_configLoader(0)
 {
     m_actions = new KActionCollection(this);
 
@@ -149,7 +147,7 @@ void WallpaperInterface::syncWallpaperPackage()
     }
 
     m_qmlObject->rootContext()->setContextProperty(QStringLiteral("wallpaper"), this);
-    m_qmlObject->setSource(QUrl::fromLocalFile(m_pkg.filePath("mainscript")));
+    m_qmlObject->setSource(m_pkg.fileUrl("mainscript"));
 
     const QString rootPath = m_pkg.metadata().value(QStringLiteral("X-Plasma-RootPath"));
     if (!rootPath.isEmpty()) {
@@ -199,13 +197,13 @@ QList<QAction *> WallpaperInterface::contextualActions() const
 
 bool WallpaperInterface::supportsMimetype(const QString &mimetype) const
 {
-    return KPluginMetaData::readStringList(m_pkg.metadata().rawData(), "X-Plasma-DropMimeTypes").contains(mimetype);
+    return KPluginMetaData::readStringList(m_pkg.metadata().rawData(), QStringLiteral("X-Plasma-DropMimeTypes")).contains(mimetype);
 }
 
 void WallpaperInterface::setUrl(const QUrl &url)
 {
     if (m_qmlObject->rootObject()) {
-        QMetaObject::invokeMethod(m_qmlObject->rootObject(), QStringLiteral("setUrl").toLatin1(), Qt::DirectConnection, Q_ARG(QVariant, QVariant::fromValue(url)));
+        QMetaObject::invokeMethod(m_qmlObject->rootObject(), "setUrl", Qt::DirectConnection, Q_ARG(QVariant, QVariant::fromValue(url)));
     }
 }
 
@@ -220,14 +218,9 @@ void WallpaperInterface::setAction(const QString &name, const QString &text, con
         action = new QAction(text, this);
         m_actions->addAction(name, action);
 
-        if (!m_actionSignals) {
-            m_actionSignals = new QSignalMapper(this);
-            connect(m_actionSignals, SIGNAL(mapped(QString)),
-                    this, SLOT(executeAction(QString)));
-        }
-
-        connect(action, SIGNAL(triggered()), m_actionSignals, SLOT(map()));
-        m_actionSignals->setMapping(action, name);
+        connect(action, &QAction::triggered, this, [this, name] {
+            executeAction(name);
+        });
     }
 
     if (!icon.isEmpty()) {
@@ -247,11 +240,7 @@ void WallpaperInterface::removeAction(const QString &name)
     QAction *action = m_actions->action(name);
 
     if (action) {
-        if (m_actionSignals) {
-            m_actionSignals->removeMappings(action);
-        }
         m_actions->removeAction(action);
-
         delete action;
     }
     setProperty("contextualActions", QVariant::fromValue(contextualActions()));
@@ -265,7 +254,8 @@ QAction *WallpaperInterface::action(QString name) const
 void WallpaperInterface::executeAction(const QString &name)
 {
     if (m_qmlObject->rootObject()) {
-        QMetaObject::invokeMethod(m_qmlObject->rootObject(), QString("action_" + name).toLatin1(), Qt::DirectConnection);
+        const QByteArray actionName("action_" + name.toUtf8());
+        QMetaObject::invokeMethod(m_qmlObject->rootObject(), actionName.constData(), Qt::DirectConnection);
     }
 }
 
